@@ -221,7 +221,6 @@ May either be a string or a list of strings.")
     (epg-context-set-progress-callback context
 				       #'epa-progress-callback-function)
     (epg-context-set-armor context epa-armor)
-    (epg-context-set-textmode context epa-textmode)
     (condition-case error
 	(setq string
 	      (epg-encrypt-string
@@ -231,7 +230,8 @@ May either be a string or a list of strings.")
 		 (epa-file--encode-coding-string (buffer-substring start end)
 						 coding-system))
 	       (if (or epa-file-select-keys
-		       (not (local-variable-p 'epa-file-encrypt-to)))
+		       (not (local-variable-p 'epa-file-encrypt-to
+					      (current-buffer))))
 		   (epa-select-keys
 		    context
 		    "Select recipents for encryption.
@@ -266,7 +266,8 @@ If no one is selected, symmetric encryption will be performed.  "
   (if (and buffer-file-name
 	   (string-match epa-file-name-regexp buffer-file-name)
 	   epa-file-inhibit-auto-save)
-      (auto-save-mode 0)))
+      (auto-save-mode 0))
+  (set-buffer-modified-p nil))
 
 (defun epa-file-select-keys ()
   "Select recipients for encryption."
