@@ -1193,6 +1193,11 @@ This function is for internal use only."
 	   (file-exists-p (epg-context-output-file context)))
       (delete-file (epg-context-output-file context))))
 
+(eval-and-compile
+  (if (fboundp 'decode-coding-string)
+      (defalias 'epg--decode-coding-string 'decode-coding-string)
+    (defalias 'epg--decode-coding-string 'identity)))
+
 (defun epg--status-USERID_HINT (context string)
   (if (string-match "\\`\\([^ ]+\\) \\(.*\\)" string)
       (let* ((key-id (match-string 1 string))
@@ -1218,6 +1223,17 @@ This function is for internal use only."
 (defun epg--status-NEED_PASSPHRASE_PIN (context string)
   (setq epg-key-id 'PIN))
 
+(eval-and-compile
+  (if (fboundp 'clear-string)
+      (defalias 'epg--clear-string 'clear-string)
+    (defun epg--clear-string (string)
+      (fillarray string 0))))
+
+(eval-and-compile
+  (if (fboundp 'encode-coding-string)
+      (defalias 'epg--encode-coding-string 'encode-coding-string)
+    (defalias 'epg--encode-coding-string 'identity)))
+
 (defun epg--status-GET_HIDDEN (context string)
   (when (and epg-key-id
 	     (string-match "\\`passphrase\\." string))
@@ -1275,9 +1291,15 @@ This function is for internal use only."
     (y-or-n-p (if entry (cdr entry) (concat string "? ")))))
 
 (defun epg--prompt-GET_BOOL-untrusted_key.override (context string)
-  (y-or-n-p (if (equal (car epg-last-status) "USERID_HINT")
-		(format "Untrusted key %s.  Use anyway? "
-			(cdr epg-last-status))
+  (y-or-n-p (if (and (equal (car epg-last-status) "USERID_HINT")
+		     (string-match "\\`\\([^ ]+\\) \\(.*\\)"
+				   (cdr epg-last-status)))
+		(let* ((key-id (match-string 1 (cdr epg-last-status)))
+		       (user-id (match-string 2 (cdr epg-last-status)))
+		       (entry (assoc key-id epg-user-id-alist)))
+		  (if entry
+		      (setq user-id (cdr entry)))
+		  (format "Untrusted key %s %s.  Use anyway? " key-id user-id))
 	      "Use untrusted key anyway? ")))
 
 (defun epg--status-GET_BOOL (context string)
@@ -1491,6 +1513,15 @@ This function is for internal use only."
 	       (if (consp (epg-context-progress-callback context))
 		   (cdr (epg-context-progress-callback context))))))
 
+(defun epg--status-ENC_TO (context string)
+  (if (string-match "\\`\\([0-9A-Za-z]+\\) \\([0-9]+\\) \\([0-9]+\\)" string)
+      (epg-context-set-result-for
+       context 'encrypted-to
+       (cons (list (match-string 1 string)
+		   (string-to-number (match-string 2 string))
+		   (string-to-number (match-string 3 string)))
+	     (epg-context-result-for context 'encrypted-to)))))
+
 (defun epg--status-DECRYPTION_FAILED (context string)
   (epg-context-set-result-for context 'decryption-failed t))
 
@@ -1807,72 +1838,60 @@ NAME is either a string or a list of strings."
       (setq pointer (cdr pointer)))
     keys))
 
-(if (fboundp 'make-temp-file)
-    (defalias 'epg--make-temp-file 'make-temp-file)
-  (defvar temporary-file-directory)
-  ;; stolen from poe.el.
-  (defun epg--make-temp-file (prefix)
-    "Create a temporary file.
+(eval-and-compile
+  (if (fboundp 'make-temp-file)
+      (defalias 'epg--make-temp-file 'make-temp-file)
+    (defvar temporary-file-directory)
+    ;; stolen from poe.el.
+    (defun epg--make-temp-file (prefix)
+      "Create a temporary file.
 The returned file name (created by appending some random characters at the end
 of PREFIX, and expanding against `temporary-file-directory' if necessary),
 is guaranteed to point to a newly created empty file.
 You can then use `write-region' to write new data into the file."
-    (let (tempdir tempfile)
-      (setq prefix (expand-file-name prefix
-				     (if (featurep 'xemacs)
-					 (temp-directory)
-				       temporary-file-directory)))
-      (unwind-protect
-	  (let (file)
-	    ;; First, create a temporary directory.
-	    (while (condition-case ()
-		       (progn
-			 (setq tempdir (make-temp-name
-					(concat
-					 (file-name-directory prefix)
-					 "DIR")))
-			 ;; return nil or signal an error.
-			 (make-directory tempdir))
-		     ;; let's try again.
-		     (file-already-exists t)))
-	    (set-file-modes tempdir 448)
-	    ;; Second, create a temporary file in the tempdir.
-	    ;; There *is* a race condition between `make-temp-name'
-	    ;; and `write-region', but we don't care it since we are
-	    ;; in a private directory now.
-	    (setq tempfile (make-temp-name (concat tempdir "/EMU")))
-	    (write-region "" nil tempfile nil 'silent)
-	    (set-file-modes tempfile 384)
-	    ;; Finally, make a hard-link from the tempfile.
-	    (while (condition-case ()
-		       (progn
-			 (setq file (make-temp-name prefix))
-			 ;; return nil or signal an error.
-			 (add-name-to-file tempfile file))
-		     ;; let's try again.
-		     (file-already-exists t)))
-	    file)
-	;; Cleanup the tempfile.
-	(and tempfile
-	     (file-exists-p tempfile)
-	     (delete-file tempfile))
-	;; Cleanup the tempdir.
-	(and tempdir
-	     (file-directory-p tempdir)
-	     (delete-directory tempdir))))))
-
-(if (fboundp 'clear-string)
-    (defalias 'epg--clear-string 'clear-string)
-  (defun epg--clear-string (string)
-    (fillarray string 0)))
-
-(if (fboundp 'encode-coding-string)
-    (defalias 'epg--encode-coding-string 'encode-coding-string)
-  (defalias 'epg--encode-coding-string 'identity))
-
-(if (fboundp 'decode-coding-string)
-    (defalias 'epg--decode-coding-string 'decode-coding-string)
-  (defalias 'epg--decode-coding-string 'identity))
+      (let (tempdir tempfile)
+	(setq prefix (expand-file-name prefix
+				       (if (featurep 'xemacs)
+					   (temp-directory)
+					 temporary-file-directory)))
+	(unwind-protect
+	    (let (file)
+	      ;; First, create a temporary directory.
+	      (while (condition-case ()
+			 (progn
+			   (setq tempdir (make-temp-name
+					  (concat
+					   (file-name-directory prefix)
+					   "DIR")))
+			   ;; return nil or signal an error.
+			   (make-directory tempdir))
+		       ;; let's try again.
+		       (file-already-exists t)))
+	      (set-file-modes tempdir 448)
+	      ;; Second, create a temporary file in the tempdir.
+	      ;; There *is* a race condition between `make-temp-name'
+	      ;; and `write-region', but we don't care it since we are
+	      ;; in a private directory now.
+	      (setq tempfile (make-temp-name (concat tempdir "/EMU")))
+	      (write-region "" nil tempfile nil 'silent)
+	      (set-file-modes tempfile 384)
+	      ;; Finally, make a hard-link from the tempfile.
+	      (while (condition-case ()
+			 (progn
+			   (setq file (make-temp-name prefix))
+			   ;; return nil or signal an error.
+			   (add-name-to-file tempfile file))
+		       ;; let's try again.
+		       (file-already-exists t)))
+	      file)
+	  ;; Cleanup the tempfile.
+	  (and tempfile
+	       (file-exists-p tempfile)
+	       (delete-file tempfile))
+	  ;; Cleanup the tempdir.
+	  (and tempdir
+	       (file-directory-p tempdir)
+	       (delete-directory tempdir)))))))
 
 (defun epg--args-from-sig-notations (notations)
   (apply #'nconc
